#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
############################################################################
#    Copyright (C) 2008 by Sergio Jovani <lesergi@gmail.com>               #
#                                                                          #
#    This program is free software; you can redistribute it and/or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################
import sys
import os
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from string import *
import gdata.youtube
import gdata.youtube.service
import re
import urllib2
import httplib
import socket
import locale
import user
import urllib
import time
import tempfile
import os.path
import subprocess
import webbrowser


# VERSION
version='0.3'


if os.name == "nt":
	creationflags=134217728
else:
	creationflags=0

# REGEXP VALID URL
_VALID_URL="^(?:http://)(?:[a-z]+\.)?(?:youtube\.[a-z]+/watch(?:\.php)?\?)(?:.*&)?(?:v=)([0-9A-Za-z-_]+)(?:&.*)?"

# GET DIRECTORY OF PROGRAM
workdir=os.path.abspath(os.path.dirname(sys.argv[0]))

def Image(img):
	return os.path.join(workdir, 'img', img)

# CONFIGURATION FILE PATH
ConfigFile=os.path.join(user.home,".elltube")

# CODIFICATION

#codification=locale.getdefaultlocale()[1]
# changed in version 0.2.2
try:
	codification=locale.getpreferredencoding()
except:
	codification="utf-8"

# HTTP HEADERS
if os.name == "posix":
	std_headers = {
		'User-Agent': 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1',
		'Accept-Charset': codification+';q=0.7,*;q=0.7',
		'Accept': 'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5',
		'Accept-Language': 'en-us,en;q=0.5',
	}	
elif os.name == "nt":
	std_headers = {
		'User-Agent': 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1',
		'Accept-Charset': codification+';q=0.7,*;q=0.7',
		'Accept': 'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5',
		'Accept-Language': 'en-us,en;q=0.5',
	}


# FILE MANAGERS
if os.name == "posix":
	filemanagers=["xdg-open","dolphin","d3lphin","konqueror","gnome-open","nautilus","thunar","rox"]
elif os.name == "nt":
	filemanagers=['explorer.exe']

# PATH LIST
initpath = os.environ["PATH"].split(":")
PATH=[]
for directory in initpath:
	PATH.append(os.path.normpath(directory))

i=0
for directory in PATH:
	if PATH.count(directory) > 1:
		del(PATH[i])
	i+=1
	
	
# FORMATS INDEX
IndexAVI=1
IndexMPEG=2
IndexMP4=3
Index3GP=4
IndexMP3=5
IndexWAV=6
IndexAMR=7
ListAudio=[IndexMP3, IndexWAV, IndexAMR]

# LOAD CONFIGURATION
def loadconf(name):
	if os.access(ConfigFile, os.R_OK):
		fConf=open(ConfigFile, "r")
		ConfData=fConf.read()
		fConf.close()
		ConfLines=ConfData.splitlines()
		for line in ConfLines:
			if line.startswith(name+'='):
				return line.split('=',1)[-1]
# SAVE CONFIGURATION
def saveconf(name, value):
	ConfLines=[]
	if os.path.lexists(ConfigFile):
		if os.access(ConfigFile, os.W_OK):
			fConf=open(ConfigFile, "r")
			ConfData=fConf.read()
			fConf.close()
			ConfLines=ConfData.splitlines()
			i=0
			for line in ConfLines:
				if line.startswith(name+'='):
					ConfLines[i]=''
				i+=1

	ConfLines.append(name+'='+value)

	fConf=open(ConfigFile, "w")

	for line in ConfLines:
		if line:
			fConf.write(line+'\n')
	fConf.close()

def verbose(msg):
	if len(sys.argv) > 1:
		if sys.argv[1]=="-v":
			print msg


class MainWindow(QMainWindow):
	def __init__(self, parent = None):
		QMainWindow.__init__(self, parent)
		
		# SIZE AND SHOW AT MIDDLE
		self.resize(600,450)
		screen = QDesktopWidget().screenGeometry()
		size =  self.geometry()
		self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)
		self.setMaximumWidth(screen.width())
		self.setMaximumHeight(screen.height())

		# TITLE AND WINDOW ICON
		self.setWindowTitle(QString(self.tr("Elltube - Download YouTube videos!")))
		self.icon = QIcon()
		self.icon.addPixmap(QPixmap(os.path.join(workdir,"img","elltube.png")))
		self.setWindowIcon(self.icon)

		# LAYOUTS
		self.widget=QWidget()
		self.glayout = QGridLayout(self.widget)
		self.hlayoutURL = QHBoxLayout()
		self.hlayoutData = QHBoxLayout()
		self.hlayoutFormat = QHBoxLayout()
		self.hlayoutSave = QHBoxLayout()
		self.hlayoutProgress = QHBoxLayout()
		self.hlayoutQuit = QHBoxLayout()
		self.vlayout = QVBoxLayout()
		self.glayout.addLayout(self.vlayout,0,0)

		# URL SECTION
		self.lURL=QLabel(QString(self.tr("YouTube URL")))
		self.hlayoutURL.addWidget(self.lURL)
		self.leURL=QLineEdit()
		self.hlayoutURL.addWidget(self.leURL)
		self.pbPaste=QPushButton(QString(self.tr("Paste")))
		self.pbPaste.setIcon(QIcon(os.path.join(workdir,'img','paste.png')))
		self.hlayoutURL.addWidget(self.pbPaste)
		self.vlayout.addLayout(self.hlayoutURL)

		# DATA AND INFORMATION SECTION
		self.lData=QLabel()
		self.lData.hide()
		self.tbData=QTextBrowser()
		tbDataMsg=QString(self.tr("Paste YouTube URL above this"))
		self.tbData.setText(QString("<center><b>%s</b></center>" % tbDataMsg))
		self.hlayoutData.addWidget(self.lData)
		self.hlayoutData.addWidget(self.tbData)
		self.vlayout.addLayout(self.hlayoutData)

		# FORMAT SECTION
		self.lFormat=QLabel(QString(self.tr("Format")))
		self.hlayoutFormat.addWidget(self.lFormat)
		self.cbFormat=QComboBox(self)
		self.cbFormat.insertItem(0, QString("FLV"))
		
		self.ffmpegsup=False	
		if os.name == "posix":
			# GET PATH AND CHECK IF FFMPEG INSTALLED
			PATH = os.environ["PATH"].split(":")
			self.executable="ffmpeg"
			

			for directory in PATH:
				if os.access(os.path.join(directory, self.executable), os.X_OK):
					self.ffmpegsupport()
					self.ffmpegsup=True
					break
				
			if not self.ffmpegsup:
				self.cbFormat.insertItem(1, QString(self.tr("Others")))
				self.connect(self.cbFormat, SIGNAL("activated(int)"), self.installffmpeg)
				self.cbFormat.setCurrentIndex(0)
				
		elif os.name == "nt":
			self.executable=os.path.join(workdir, "ffmpeg.exe")
			if os.access(os.path.join(workdir, "ffmpeg.exe"), os.X_OK):
				self.ffmpegsupport()
				self.ffmpegsup=True

		
		self.hlayoutFormat.addWidget(self.cbFormat)
		self.spacerItem=QSpacerItem(0,0,QSizePolicy.Expanding,QSizePolicy.Minimum)
		self.hlayoutFormat.insertItem(0,self.spacerItem)
		self.hlayoutFormat.insertItem(-1,self.spacerItem)
		self.vlayout.addLayout(self.hlayoutFormat)

		# SAVE PATH SECTION
		self.lSave=QLabel(QString(self.tr("Save to")))
		self.hlayoutSave.addWidget(self.lSave)
		self.leSave=QLineEdit(QString(user.home))
		self.leSave.setReadOnly(True)
		self.hlayoutSave.addWidget(self.leSave)
		self.pbSave=QPushButton(QString(self.tr("Select Place")))
		self.pbSave.setIcon(QIcon(os.path.join(workdir,'img','folder.png')))
		self.hlayoutSave.addWidget(self.pbSave)
		self.pbOpen=QPushButton(self.tr("Open"))
		self.pbOpen.setIcon(QIcon(os.path.join(workdir,'img','open.png')))
		self.hlayoutSave.addWidget(self.pbOpen)
		self.vlayout.addLayout(self.hlayoutSave)

		# DOWNLOAD SECTION
		self.pbDownload=QPushButton(QString(self.tr("Download!")))
		self.pbDownload.setIcon(QIcon(os.path.join(workdir,'img','save.png')))
		self.pbDownload.setDisabled(True)
		self.vlayout.addWidget(self.pbDownload)

		# PROGRESSBAR SECTION
		self.progressbar=QProgressBar()
		self.progressbar.setValue(0)
		self.hlayoutProgress.addWidget(self.progressbar)
		self.pbCancel=QPushButton(QString(self.tr("Cancel")))
		self.pbCancel.setIcon(QIcon(os.path.join(workdir, 'img', 'cancel.png')))
		self.hlayoutProgress.addWidget(self.pbCancel)
		self.vlayout.addLayout(self.hlayoutProgress)

		
		# PROCESS INFO
		self.lProcess=QLabel()
		self.vlayout.addWidget(self.lProcess)
		self.processmsg(self.tr("Paste YouTube URL on top"))
		
		# QUIT SECTION
		self.cbQuit=QCheckBox(QString(self.tr("Quit when all operations finish")))
		self.hlayoutQuit.addWidget(self.cbQuit)
		self.hlayoutQuit.insertItem(0,self.spacerItem)
		self.hlayoutQuit.insertItem(-1,self.spacerItem)
		self.vlayout.addLayout(self.hlayoutQuit)

		# SHOW/HIDE WIDGETS
		self.cbQuit.hide()
		self.pbCancel.hide()
		self.progressbar.hide()
		
		# CONNECTIONS
		# WHEN URL CHANGES, SEARCH VIDEO
		self.connect(self.leURL, SIGNAL("textChanged(QString)"), self.getdata)
		# PASTE IN URL CLIPBOARD CONTENT
		self.connect(self.pbPaste, SIGNAL("clicked()"), self.pasteurl)
		# SELECT PLACE TO DOWNLOAD
		self.connect(self.pbSave, SIGNAL("clicked()"), self.saveto)
		# OPEN PLACE TO DOWNLOAD
		self.connect(self.pbOpen, SIGNAL("clicked()"), self.opendir)
		# DOWNLOAD VIDEO
		self.connect(self.pbDownload, SIGNAL("clicked()"), self.download)
		# CANCEL VIDEO DOWNLOAD
		self.connect(self.pbCancel, SIGNAL("clicked()"), self.cancel)

		self.connect(QApplication.instance(), SIGNAL('focusChanged(QWidget *,QWidget *)'), self.focusChanged)


		self.setCentralWidget(self.widget)
	

		# STATUSBAR
		self.statusbar=QStatusBar()
		self.setStatusBar(self.statusbar)


		# MENUBAR
		# EXIT ACTION
		exit=QAction(QIcon(QString(os.path.join(workdir,'img','exit.png'))), QString(self.tr('Exit')), self)
		exit.setShortcut('Ctrl+Q')
		exit.setStatusTip(QString(self.tr('Exit Elltube')))
		self.connect(exit, SIGNAL('triggered()'), SLOT('close()'))

		# LOGIN ACTION
		login=QAction(QIcon(QString(os.path.join(workdir,'img','login.png'))), QString(self.tr('Login')), self)
		login.setStatusTip(QString(self.tr('Login as YouTube user')))
		self.connect(login, SIGNAL('triggered()'), self.logindlg)

		# SETTINGS ACTION
		settings=QAction(QIcon(QString(os.path.join(workdir,'img','settings.png'))), QString(self.tr('Settings')), self)
		settings.setStatusTip(QString(self.tr('General settings')))
		self.connect(settings, SIGNAL('triggered()'), self.settingsdlg)

		# ABOUT ACTION
		about=QAction(QIcon(QString(os.path.join(workdir,'img','elltube.png'))), QString(self.tr('About')), self)
		about.setStatusTip(QString(self.tr('About Elltube')))
		self.connect(about, SIGNAL('triggered()'), self.about)

		# ABOUTQT ACTION
		aboutqt=QAction(QString(self.tr('About Qt')), self)
		aboutqt.setStatusTip(QString(self.tr('About Qt')))
		self.connect(aboutqt, SIGNAL('triggered()'), self.aboutqt)

		# MENUBAR BUILD
		menubar=self.menuBar()
		file=menubar.addMenu(QString(self.tr('&File')))
		file.addAction(exit)
		tools=menubar.addMenu(QString(self.tr('&Tools')))
		tools.addAction(settings)
		tools.addAction(login)
		help=menubar.addMenu(QString(self.tr('&Help')))
		help.addAction(about)
		help.addAction(aboutqt)


		self.threadGetData=None

		# LOAD CONFIGURATION
		self.loadstart()
	
	def autoPaste(self):
		# PASTE CLIPBOARD IF VALID CONTENT
		clipboard=QApplication.clipboard()
		clipboard=unicode(clipboard.text())
		
		validurl=re.match(_VALID_URL, clipboard)
		
		if validurl:
			if not clipboard == str(self.leURL.text()):
				self.pasteurl()


	def focusChanged(self, old, new):
		if ( not old and new):
			if not self.leURL.isReadOnly():
				self.autoPaste()

	# IF CONVERTING FAILS
	def openlog(self):
		DlgOpenlog=OpenLog(self)
		DlgOpenlog.exec_()

			
	# IF THERE IS NOT FFMPEG
	def installffmpeg(self, formatindex):
		if formatindex == 1:
			msgbox=QMessageBox.critical(self, QString(self.tr("FFMPEG required!")), QString(self.tr("Please, install 'ffmpeg' program.")), QMessageBox.Ok)
			self.cbFormat.setCurrentIndex(0)
		
	# PUT MESSAGE IN PROCESS LABEL
	def processmsg(self, msg):
		self.lProcess.setText(QString("<center><h3><b>"+msg+"</b></h3></center"))
			
	# ADD FFMPEG SUPPORT IN FORMAT LIST
	def ffmpegsupport(self):
		verbose("Detecting codecs from ffmpeg")
		#DETECT CODECS
		output=subprocess.Popen([self.executable, "-formats"], stderr=subprocess.STDOUT, stdout=subprocess.PIPE, creationflags=creationflags).communicate()[0]
		outcodecs=output.split("Codecs:")
		outcodecs=outcodecs[1].split("\n")
		outformats=output.split("File formats:")
		outformats=outformats[1].split("\n")
		result=None
		codecs=[]
		namecodecs=[]
		codecs.append("xvid")
		codecs.append("mp3")
		codecs.append("aac")
		codecs.append("mpeg4")
		codecs.append("h263")
		codecs.append("amr")

		nameformats=[]
		
		for codec in codecs:
			_VIDEO_CODEC="^(?: )*(?:[DVSAT]*E+[DVSAT]*)(?: )*([0-9a-z]*%s[0-9a-z]*)(?: )*(?:.*)$" % codec
		
		
			for line in outcodecs:
				results=re.match(_VIDEO_CODEC, line)
			
				if results:
					result=results.group(1).strip()
					
					if ( result == codec or result == "lib"+codec):
						break
		
			namecodecs.append(result)
			result=None


		result=None
		_VIDEO_FORMAT="^(?: )*(?:D)?(?:E)(?: )+([0-9a-z]+)(?: )*(?:.)*$"
		for line in outformats:
			result=re.match(_VIDEO_FORMAT, line)

			if result:
				nameformats.append(result.group(1).strip())
			result=None
			
			
		self.CodecXVID=namecodecs[0]
		self.CodecMP3=namecodecs[1]
		self.CodecAAC=namecodecs[2]
		self.CodecMPEG4=namecodecs[3]
		self.CodecH263=namecodecs[4]
		self.CodecAMR=namecodecs[5]

		verbose("Codecs detected: %s" % namecodecs)
		verbose("Creating video formats list")

		
		

		if (self.CodecXVID and self.CodecMP3 or "avi" in nameformats):
			self.cbFormat.insertItem(IndexAVI,QString(self.tr("AVI")))
		
		if "mpeg" in nameformats:
			self.cbFormat.insertItem(IndexMPEG,QString(self.tr("MPEG")))
		
		if (self.CodecMPEG4 or "mp4" in nameformats):
			self.cbFormat.insertItem(IndexMP4,QString(self.tr("MP4")))
			
		if (self.CodecH263 and self.CodecAAC or "3gp" in nameformats):
			self.cbFormat.insertItem(Index3GP,QString(self.tr("3GP")))
			
		if (self.CodecMP3 or "mp3" in nameformats):
			self.cbFormat.insertItem(IndexMP3,QString(self.tr("MP3 (audio only)")))

		if "wav" in nameformats:
			self.cbFormat.insertItem(IndexWAV, QString(self.tr("WAV (audio only)")))
			
		if (self.CodecAMR or "amr" in nameformats):
			self.cbFormat.insertItem(IndexAMR,QString(self.tr("AMR (audio only)")))

	# OPEN DIRECTORY
	def opendir(self):
		
		dirpath=unicode(self.leSave.text())
		filemanager=None
		openconf=loadconf("FileManager")

		if not openconf:
			if os.name == "posix":
				for manager in filemanagers:
					for directory in PATH:
						if os.access(os.path.join(directory, manager), os.X_OK):
							filemanager=manager
							break
					if filemanager:
						break
			elif os.name == "nt":
				filemanager="explorer.exe"
		else:
			filemanager=openconf
		subprocess.Popen([filemanager, dirpath.encode(codification)])

		verbose("Opening download directory %s with %s" % (dirpath.encode(codification), filemanager))

		

	# SAVE CURRENT STATE OF WIDGETS
	def savequit(self):
		verbose("Saving configuration")

		Format=str(self.cbFormat.currentIndex())
		Path=self.leSave.text()
		Quit=str(self.cbQuit.isChecked())

		saveconf("Format", Format)
		saveconf("Path", Path)
		saveconf("Quit", Quit)

		# IF NOT REMEMBER, DELETE USER AND PASSWORD
		Remember=loadconf("Remember")
		if Remember == "False":
			saveconf("User", "")
			saveconf("Password", "")

	# LOAD SAVED STATE OF WIDGETS
	def loadstart(self):
			verbose("Loading configuration")
			Format=loadconf("Format")
			Path=loadconf("Path")
			Quit=loadconf("Quit")

			if Format:
				if self.ffmpegsup:
					self.cbFormat.setCurrentIndex(int(Format))
				else:
					self.cbFormat.setCurrentIndex(0)
			if (Path and os.path.exists(Path)):
				self.leSave.setText(QString(Path))
			if Quit == "True":
				self.cbQuit.setChecked(True)
			else:
				self.cbQuit.setChecked(False)
	
	# PASTE FROM CLIPBOARD TO URL INPUT
	def pasteurl(self):
		self.leURL.selectAll()
		self.leURL.paste()

	# INITIALIZE VIDEO DATA SEARCH
	def getdata(self):
		verbose("Searching info")
		self.leURL.setReadOnly(True)
		self.url=self.leURL.text()

		self.lData.hide()
		self.pbDownload.setDisabled(True)
		self.pbPaste.setDisabled(True)
		SearchingMsg=self.tr("Searching video...")
		self.tbData.setText(QString("<center><b>%s</b></center>" % SearchingMsg))
		self.processmsg(self.tr("Searching video for %1").arg(self.url))
		if not self.threadGetData:
			self.threadGetData = GetData()
			# SEND TO PUTINFO DATA ARRAY FROM GETDATA THREAD
			self.connect(self.threadGetData, SIGNAL("data"), self.putinfo)
		self.threadGetData.getdata(self.url)
	
	# PUT IN WIDGET GET VIDEO DATA
	def putinfo(self, data):
		# VIDEO NOT FOUND
		if data==None:
			verbose("Video not found")
			NotFoundMsg=self.tr("Video not found!")
			self.tbData.setText(QString("<center><b>%s</b></center>" % NotFoundMsg))
			self.processmsg(NotFoundMsg)
		# VIDEO FOUND
		else:
			# GET DATA FROM ARRAY
			self.videoid=data[0]
			self.title=unicode(data[1],"utf-8")
			description=unicode(data[2],"utf-8")
			thumbfile=data[3]
			duration=data[4]
			self.lData.show()
			# SHOW VIDEO THUMBNAIL
			self.thumbnail=QPixmap()
			self.thumbnail.load(thumbfile)
			self.lData.setPixmap(self.thumbnail)
			# REMOVE THUMBNAIL LOCALE FILE
			os.remove(thumbfile)
			DurationMsg=self.tr("Duration: %1 min").arg(duration)
			self.tbData.setText(QString("<b>%s</b><br><br>%s<br><br>%s" % (self.title, description, DurationMsg)))
			VideoFoundMsg=self.tr("Video found!")
			self.processmsg(VideoFoundMsg)
			self.pbDownload.setEnabled(True)

		self.leURL.setReadOnly(False)
		self.pbPaste.setEnabled(True)
	
	# SELECT DOWNLOAD DIRECTORY
	def saveto(self):
		filedialog=QFileDialog.getExistingDirectory(self, QString(self.tr("Select where to save the video")), QString(self.leSave.text()), QFileDialog.ShowDirsOnly)
		if os.name == "nt":
			filedialog=unicode(filedialog)
		
		filedialog=os.path.normcase(filedialog)
		if filedialog:
			verbose("Download directory: %s" % filedialog)
			self.leSave.setText(QString(filedialog))
		else:
			verbose("Any directory specified")

	# BEGIN DOWNLOAD PROCESS
	def download(self):
		# LIMIT 40 CHARS IN FILENAMES
		self.filename=self.title[:40]
		self.filename=self.filename.replace(":"," ").replace("/","-").replace("\\","-")
	
		self.filename_flv=self.filename+".flv"
	
		self.path=unicode(self.leSave.text())

		self.pathfile=os.path.join(self.path,self.filename_flv)
		
		# IF WE HAVE ACCESS AND IF FILE EXISTS
		if os.access(self.path, os.W_OK):
			if os.path.lexists(self.pathfile):
				# FOLDER EXISTS
				if os.path.isdir(self.pathfile):
					msgbox=QMessageBox(self)
					msgbox.setText(QString(self.tr("A folder named %1 already exists. Please, choose other folder.").arg(self.filename_flv)))
					msgbox.setWindowTitle("Error!")
					msgbox.setIcon(QMessageBox.Critical)
					msgbox.exec_()
					# OPEN SELECT DIRECTORY DIALOG
					self.saveto()
					return None
				# FILE EXISTS
				if os.path.isfile(self.pathfile):
					msgbox=QMessageBox(self)
					msgbox.setText(QString(self.tr("A file named %1 already exists. Are you sure you want to overwrite it?").arg(self.filename_flv)))
					msgbox.setWindowTitle(QString(self.tr("File exists!")))
					msgbox.setIcon(QMessageBox.Question)
					nobutton=msgbox.addButton(QString(self.tr("No")), QMessageBox.ActionRole)
					overbutton=msgbox.addButton(QString(self.tr("Overwrite")), QMessageBox.ActionRole)
					msgbox.exec_()

					# IF WE WANT TO OVERWRITE
					if msgbox.clickedButton() == nobutton:
						# OPEN SELECT DIRECTORY DIALOG
						self.saveto()
						return None
		# IF WE DON'T HAVE ACCESS
		else:
			msgbox=QMessageBox(self)
			msgbox.setText(QString(self.tr("You don't have permissions in order to download the video in specified path. Please, choose other one.")))
			msgbox.setWindowTitle(QString(self.tr("Error!")))
			msgbox.setIcon(QMessageBox.Critical)
			msgbox.exec_()
			# OPEN SELECT DIRECTORY DIALOG
			self.saveto()
			return None

		# PREPARE WIDGETS STATE
		self.pbDownload.hide()
		self.progressbar.setValue(0)
		self.progressbar.show()
		self.pbCancel.show()
		self.cbQuit.show()
		self.pbCancel.setDisabled(True)
		self.leURL.setReadOnly(True)
		self.pbPaste.setDisabled(True)
		self.pbSave.setDisabled(True)
		# THREAD IN ORDER TO GET FLV URL
		self.threadGetFlvURL = GetFlvURL(self.videoid)
		self.processmsg(self.tr("Locating FLV file"))
		# WHEN LOGIN IS REQUIRED BUT YOUTUBE IS NOT AVAIBLE
		self.connect(self.threadGetFlvURL, SIGNAL("youtubeout"), self.youtubeout)
		# DOWNLOAD WHEN FLV URL IS FOUND
		self.connect(self.threadGetFlvURL, SIGNAL("flvurl"), self.GetFile)
		# SHOW LOGIN DIALOG WHEN FLV URL REQUIRES
		self.connect(self.threadGetFlvURL, SIGNAL("login"), self.needlogin)
		# SET STATUSBAR FROM THREAD
		self.connect(self.threadGetFlvURL, SIGNAL('processmsg'), self.processmsg)
		# ENABLE WIDGETS
		self.connect(self.threadGetFlvURL, SIGNAL('loginfailed'), self.loginfailed)

		self.threadGetFlvURL.start()
		
	# WHEN LOGIN IS REQUIRED BUT YOUTUBE IS NOT AVAIBLE
	def youtubeout(self):
		verbose("YouTube unavaible")
		self.processmsg(self.tr("Unable get FLV location. Maybe is YouTube unavaible?"))
		self.enablewidgets()
		self.progressbar.hide()
		self.pbCancel.hide()

	# WHEN LOGIN FAILED
	def loginfailed(self):
		verbose("Login failed")
		self.threadGetFlvURL.terminate()
		self.processmsg(self.tr("Login failed!"))
		self.needlogin()

	# WHEN FLV FILE IS FOUND, INITIALIZE DOWNLOAD THREAD
	def GetFile(self, url):
		self.processmsg(self.tr("Downloading %1").arg(self.filename_flv))
		# RUN THREAD IN ORDER TO DOWNLOAD FLV FILE
		self.threadDownload = Download(url, self.filename, self.path)
		# WHEN DOWNLOADED FINISHES, SEND TO DOWNLOADED RETURN CODE
		self.connect(self.threadDownload, SIGNAL("downloaded"), self.downloaded)
		# UPDATE PROGRESSBAR
		self.connect(self.threadDownload, SIGNAL("progress"), self.updateprogress)
		# UPDATE PROCESS MESSAGES
		self.connect(self.threadDownload, SIGNAL('processmsg'), self.processmsg)

		self.threadDownload.start()

		self.pbCancel.setEnabled(True)

	def downloaded(self, value):
		# WHEN DOWNLOAD PROCESS ENDS
		self.progressbar.hide()
		self.pbCancel.hide()


		# CORRECT DOWNLOAD
		if value == 0:
			verbose("Download finished")
			format=self.cbFormat.currentIndex()
			arguments=None
			# FORMAT IS FLV, NOT CONVERT
			if format == 0:
				# IF OPEN ON FINISH IS ENABLED
				openconf=loadconf("Open")
				if openconf == "True":
					self.opendir()

				if self.cbQuit.isChecked():
					self.close()
				else:
					self.processmsg(self.tr("Download finished!"))
					self.enablewidgets()
			# FORMAT NOT FLV, CONVERT NEEDED
			else:
				if format == IndexAVI:
					self.fileformat="avi"
					if (self.CodecXVID and self.CodecMP3):
						arguments="-vcodec %s -acodec %s -aspect 1:1" % (self.CodecXVID, self.CodecMP3)
				elif format == IndexMPEG:
					self.fileformat="mpeg"
				elif format == IndexMP4:
					self.fileformat="mp4"
					if (self.CodecMPEG4 and self.CodecAAC):
						arguments="-vcodec %s -acodec %s" % (self.CodecMPEG4, self.CodecAAC)
				elif format == Index3GP:
					self.fileformat="3gp"
					if (self.CodecAAC and self.CodecH263):
						arguments="-ac 1 -acodec %s -vcodec %s -s 176x144 -r 12 -b 60" % (self.CodecAAC, self.CodecH263)
				elif format == IndexMP3:
					self.fileformat="mp3"
				elif format == IndexWAV:
					self.fileformat="wav"
				elif format == IndexAMR:
					self.fileformat="amr"
					arguments="-b 80 -ab 12200 -ar 8000 -s sqcif"

				self.fileoutput="%s.%s" % (self.filename, self.fileformat)
				
				ConvertingMsg=self.tr("Download finished! Converting video... This might takes a while.")
				GeneratingMsg=self.tr("Generating %1").arg(self.fileoutput)
				self.processmsg(ConvertingMsg+"<br>"+GeneratingMsg)
				self.cbFormat.setDisabled(True)
				
				# PREPARING EXECUTE LINE
				passarguments=[self.executable, '-i', self.filename_flv.encode(codification)]

				if arguments:
					for argument in arguments.split():
						passarguments.append(argument)

				passarguments.append(self.fileoutput.encode(codification))
				verbose("Executing %s" % passarguments)
				self.Exec("converted", passarguments)
		else:
			verbose("Download failed")
			self.enablewidgets()
			self.processmsg(self.tr("Download failed! Reason: Timed out. Check your connection."))

	# EXECUTE A COMMAND
	def Exec(self, operation, arguments):
		self.threadExec = Exec(operation, arguments)
		self.connect(self.threadExec, SIGNAL("exec_finished"), self.exec_finished)
		self.threadExec.start()

	# WHEN EXECUTE FINISHES
	def exec_finished(self, arguments):
		operation=arguments[0]
		returncode=arguments[1]

		# EXECUTE IS CONVERTED
		if operation=="converted":
			# ENDS CORRECTLY
			if returncode == 0:
				verbose("Convertig finished!")
				self.processmsg(self.tr("Converting finished!"))
				os.remove(self.filename_flv)

				# IF OPEN ON FINISH IS ENABLED
				openconf=loadconf("Open")
				if openconf == "True":
					self.opendir()

				if self.cbQuit.isChecked():
					self.close()
			# IF VIDEO HAS NOT AUDIO
			elif returncode == "noaudio":
				verbose("Converting failed! Video without audio")
				self.processmsg(self.tr("Converting failed! Selected video has not audio! Please, select a video format."))
					
				self.enablewidgets()	
			# ENDS FAILED
			else:
				verbose("Converting failed!")
				self.processmsg(self.tr("Converting failed!"))
				if os.path.exists(self.fileoutput):
					os.remove(self.fileoutput)
				self.openlog()

			self.enablewidgets()

	# SET SHOWED/HIDE WIDGET STATE LIKE START
	def enablewidgets(self):
		self.cbFormat.setEnabled(True)
		self.pbDownload.show()
		self.leURL.setReadOnly(False)
		self.pbPaste.setDisabled(False)
		self.pbSave.setDisabled(False)
		self.cbQuit.hide()

	# CANCEL DOWNLOAD
	def cancel(self):
		# TERMINATE DOWNLOAD
		verbose("\nCancelling download")
		self.threadDownload.terminate()
		# CHANGE WIDGETS STATE
		self.progressbar.hide()
		self.pbCancel.hide()
		self.cbQuit.hide()
		self.pbDownload.show()
		self.leURL.setReadOnly(False)
		self.pbPaste.setDisabled(False)
		self.pbSave.setDisabled(False)
		self.processmsg(self.tr("Download cancelled!"))
		try:
			os.remove(self.filename+".flv")
		except:
			pass

	# UPDATE PROGRESSBAR
	def updateprogress(self, value):
		self.progressbar.setValue(value)

	# NEED LOGIN IN ORDER TO DOWNLOAD VIDEO
	def needlogin(self):
		# ASK IF WANT TO LOGIN
		verbose("Login required")
		msgbox=QMessageBox(self)
		msgbox.setText(QString(self.tr("This video requires that you are identified as a user. Do you want to login?")))
		msgbox.setWindowTitle(QString(self.tr("Login required!")))
		msgbox.setIcon(QMessageBox.Question)
		nobutton=msgbox.addButton(QString(self.tr("No")), QMessageBox.ActionRole)
		yesbutton=msgbox.addButton(QString(self.tr("Yes")), QMessageBox.ActionRole)
		msgbox.exec_()

		# IF DON'T WANT, UNABLE DOWNLOAD VIDEO
		if msgbox.clickedButton() == nobutton:
			msgbox=QMessageBox.critical(self, QString(self.tr("Error!")), QString(self.tr("Unable download video. Login required.")), QMessageBox.Ok)
			self.enablewidgets()
			self.progressbar.hide()
			self.pbCancel.hide()
			self.processmsg(self.tr("Unable download video. Login required."))
		# IF WANT, SHOW LOGIN DLG AND TRY DOWNLOAD
		else:
			self.logindlg()
			self.download()
	# LOGIN DLG
	def logindlg(self):
			DlgLogin=Login(self)
			DlgLogin.exec_()

	# LOGIN DLG
	def settingsdlg(self):
			DlgSettings=Settings(self)
			DlgSettings.exec_()

	# ABOUT DLG
	def about(self):
		#dialog = QMessageBox.about(self, "About Elltube",
			#"Elltube\n" +
			#"Version %s\n\n"%(version) +
			#"YouTube Downloader and Converter.\n"+
			#u"Copyright \xa9  2008 Sergio Jovani <lesergi@gmail.com>\n"+
			#"This Software is distributed under the GPL License.\n\n"+
			#"Visit http://sourceforge.net/projects/elltube"
			#)
			
		DlgAbout=About(self)
		DlgAbout.exec_()
		
	# ABOUT QT DLG
	def aboutqt(self):
		QMessageBox.aboutQt(self)


# DOWNLOAD VIDEO THREAD
class Download(QThread):

	def __init__(self, url, title, path, parent = None):
		QThread.__init__(self, parent)
		self.url=url
		self.filename=title
		self.filext=self.filename+".flv"
		self.path=path
	# TRY DOWNLOAD
	def run(self):
		os.chdir(self.path)
		self.filesize=None
		socket.setdefaulttimeout(30)
		try:
			self.timed=time.time()
			verbose("Getting %s" % self.url)
			urllib.urlretrieve(self.url,self.filext,reporthook=self.myreporthook)
			verbose("\nDownloaded %s" % self.filext)
			self.emit(SIGNAL("downloaded"), 0)
		except socket.timeout:
			verbose("\nDownload failed!")
			self.emit(SIGNAL("downloaded"), 1)
	# SET VALUE IN PROGRESSBAR EACH BLOCK IS DOWNLOADED
	def myreporthook(self, count, blocksize, totalsize):
		if not self.filesize:
			self.filesize=float(totalsize)/1024/1024
		totaldown=(float(count)*float(blocksize)/1024/1024)
		totalsecs=time.time()-self.timed
		speed=totaldown/totalsecs*1024
		
		if not totaldown == 0:
			if not speed == 0:
				secsleft=(self.filesize*1024-totaldown*1024)/(speed)
				
				strtotaldown="%.2f" % totaldown
				strfilesize="%.2f" % self.filesize
				strspeed="%i" % speed
				DownloadingMsg=self.tr("Downloading %1").arg(self.filext)
				SpeedMsg=self.tr("Completed %1 of %2 MB at %3 KB/sec").arg(strtotaldown,strfilesize,strspeed)
				verbose("Completed %s of %s MB at %s KB/sec\033[A" % (strtotaldown,strfilesize,strspeed))
				ProgressMsg=DownloadingMsg+"<br>"+SpeedMsg
				self.emit(SIGNAL("processmsg"), ProgressMsg)
		
		self.blocks=count*blocksize*100/totalsize
		self.emit(SIGNAL("progress"), self.blocks)

# GET VIDEO DATA FROM GIVEN URL
class GetData(QThread):

	def __init__(self, parent = None):
		QThread.__init__(self, parent)

	def getdata(self, url):
		self.url=url
		self.start()

	def run(self):
		data=None
		# GET ID
		

		# CHECK IF MATCH YOUTUBE URL
		try:
			mobj = re.match(_VALID_URL, str(self.url))


		except:	
			verbose("%s is not a valid YouTube URL" % self.url)
			self.emit(SIGNAL("data"), data)
			return None

		# TRY EXTRACT VIDEO ID
		if  mobj:
			videoid = mobj.group(1)
		else:
			verbose("%s is not a valid YouTube URL" % self.url)
			self.emit(SIGNAL("data"), data)
			return None

		# GET TITLE
		yt_service = gdata.youtube.service.YouTubeService()
		verbose("Getting video info with ID %s" % videoid)
		
		try:
			entry = yt_service.GetYouTubeVideoEntry(video_id=videoid)
			verbose("Video found")
		except:
			verbose("Video not found!")
			self.emit(SIGNAL("data"), data)
			return None

		title=entry.media.title.text
		verbose("Title: %s" % title)

		# GET DESCRIPTION
		description=entry.media.description.text
		verbose("Description: %s" % description.replace("\n"," "))

		# GET THUMBNAIL
		thumbnail=entry.media.thumbnail[0].url
		thumbname=os.path.basename(thumbnail)
		tmpdir=tempfile.gettempdir()
		thumbfile=os.path.join(tmpdir,thumbname)
		thumbnaildata=urllib.urlretrieve(thumbnail, thumbfile)
		verbose("Getting thumbnail from %s and saving in %s" % (thumbnail, thumbfile))

		# GET DURATION
		duration=int(entry.media.duration.seconds)
		hours=int(duration/60)
		seconds=duration-(hours*60)
		if seconds < 10:
			seconds="0"+str(seconds)
		duration=str(hours)+":"+str(seconds)
		verbose("Duration: %s min" % duration)



		# RETURN DATA TO PUT INFO IN SCREEN
		data=[videoid, title, description, thumbfile, duration]

		self.emit(SIGNAL("data"), data)

# GET FLV URL IN ORDER TO DOWNLOAD IT
class GetFlvURL(QThread):

	def __init__(self, videoid, parent = None):
		QThread.__init__(self, parent)
		self.videoid=videoid
		self.logged=None
		# INITIALIZE COOKIES
		self.cookies=urllib2.HTTPCookieProcessor()
		self.opener = urllib2.build_opener(self.cookies)
		urllib2.install_opener(self.opener)



	# LOGIN AS USER
	def login(self):
		# LOAD CONFIG
		User=loadconf("User")
		Password=loadconf("Password")
		Google=loadconf("Google")
		
		# IF THERE IS A VALID USER
		if (User and Password):
			self.emit(SIGNAL("processmsg"),self.tr("Logging as YouTube user")) 

			# IF ACCOUNT IS NOT GOOGLE
			if Google == "False":
				verbose("Login as YouTube user")
				# POST LOGIN FORM
				_LOGIN_URL = 'http://www.youtube.com/login?next=/'
				
				login_form = {
					'current_form': 'loginForm',
					'next':		'/',
					'action_login':	'Log In',
					'username':	User,
					'password':	Password,
					} 

				request = urllib2.Request(_LOGIN_URL, urllib.urlencode(login_form), std_headers)

				# CHECK IF LOGIN IS CORRECT
				try:
					verbose("Connecting")
					login_results = urllib2.urlopen(request).read()
					if login_results.count("loginForm") > 0:
						verbose("Login failed")
						self.loginfailed()
					else:
						verbose("Logged!")
						self.emit(SIGNAL("processmsg"),self.tr("Logged!"))
						self.logged=True
				except (urllib2.URLError, httplib.HTTPException, socket.error), err:
					verbose("Error connecting")
					return None

			# IF ACCOUNT IS GOOGLE
			else:
				verbose("Login as Google user")
				# POST LOGIN FORM
				_LOGIN_URL='https://www.google.com/accounts/ServiceLoginAuth?'
				login_form = {
					'service':	'youtube',
					'hl':		'en_US',
					'passive':	'true',
					'continue':	'http://www.youtube.com/signup?hl=en_US&amp;next=/',
					'rmShown':	'1',
					'asts':		'',
					
					'Email':	User,
					'Passwd':	Password,
					} 

				request = urllib2.Request(_LOGIN_URL, urllib.urlencode(login_form), std_headers)
				
				try:
					verbose("Connecting")
					login_results = urllib2.urlopen(request).read()

					# GET REDIRECT URL
					redirects=login_results.split('<meta http-equiv="refresh" content="0; url=&#39;')[-1].split('&#3')[0].replace('&amp;','&')

					if redirects.startswith("http") == False:
						verbose("Login failed")
						self.loginfailed()
						return 0
					# FOLLOW LOGIN REDIRECTS
					try:
						verbose("Redirecting")
						login_results = urllib2.urlopen(redirects).read()
						if login_results.count('gaia_loginform') > 0:
							verbose("Login failed")
							self.loginfailed()
						else:
							verbose("Logged")
							self.emit(SIGNAL("processmsg"),self.tr("Logged!"))
							self.logged=True
					except (urllib2.URLError, httplib.HTTPException, socket.error), err:
						verbose("Error connecting")
						return None

				except (urllib2.URLError, httplib.HTTPException, socket.error):
					verbose("Error connecting")
					return None

			# CONFIRM AGE
			_AGE_URL = 'http://www.youtube.com/verify_age?next_url=/'
			age_form = {
					'next_url':		'/',
					'action_confirm':	'Confirm',
					}
			verbose("Confirming age")
			request = urllib2.Request(_AGE_URL, urllib.urlencode(age_form), std_headers)
			try:
				age_results = urllib2.urlopen(request).read()
				verbose("Age confirmed")
			except (urllib2.URLError, httplib.HTTPException, socket.error):
				verbose("Error confirming age")
				return None

		else:
			self.emit(SIGNAL("login"), None)
			self.terminate()

	# IF LOGIN IS FAILED
	def loginfailed(self):
		self.emit(SIGNAL("loginfailed"), None)
			
	# GET FLV URL
	def run(self):
		# GET FLV URL
		# SEARCH FOR T PARAM
		normalized_url = 'http://www.youtube.com/watch?v=%s' % self.videoid

		request = urllib2.Request(normalized_url, None, std_headers)

		try:
			video_webpage = urllib2.urlopen(request).read()
		except (urllib2.URLError, httplib.HTTPException, socket.error), err:
			return None
		mobj = re.search(r', "t": "([^"]+)"', video_webpage)

		if mobj == None:
			self.login()
		

			# SEARCH FOR T PARAM
			request = urllib2.Request(normalized_url, None, std_headers)


			try:
				video_webpage = urllib2.urlopen(request).read()
			except (urllib2.URLError, httplib.HTTPException, socket.error), err:
				return None
			mobj = re.search(r', "t": "([^"]+)"', video_webpage)
			

		if mobj:
			# GET FINAL FLV URL
			video_real_url = 'http://www.youtube.com/get_video?video_id=%s&t=%s' % (self.videoid, mobj.group(1))
		
			self.emit(SIGNAL("flvurl"), video_real_url)
		elif self.logged == True:
			self.emit(SIGNAL('youtubeout'), None)


# EXEC A SYSTEM PROGRAM
class Exec(QThread):
	def __init__(self, operation, arguments, parent = None):
		QThread.__init__(self, parent)
		self.operation=operation
		self.arguments=arguments

	# WHEN PROGRAM FINISHES, EMIT RETURN CODE
	def run(self):

		output=subprocess.Popen(self.arguments[:3], stderr=subprocess.STDOUT, stdout=subprocess.PIPE, creationflags=creationflags).communicate()[0]
		output=output.splitlines()
		_AUDIO_CODEC="^(?:Stream)(?:.*)(?:Audio:)(?:.*)(?:0x0000)$"
		validaudio=None
		
		for line in output:
			validaudio=re.match(_AUDIO_CODEC, line.strip())
			
			if validaudio:
				if window.cbFormat.currentIndex() in ListAudio:
					self.emit(SIGNAL("exec_finished"), [self.operation,"noaudio"])
					return None
				else:
					self.arguments.insert(-1,'-an')
		
		if os.path.exists(self.arguments[-1]):
			os.remove(self.arguments[-1])

		file=open(os.path.join(user.home,".elltube-log"),"w")
		returncode=subprocess.Popen(self.arguments, stderr=subprocess.STDOUT, stdout=file, creationflags=creationflags).wait()

		file.close()

		self.emit(SIGNAL("exec_finished"), [self.operation,returncode])

# SETTINGS DLG
class Settings(QDialog):
	def __init__(self, parent=None):
		QDialog.__init__(self, parent)

		self.setWindowTitle(self.tr("Settings"))
		
		# WIDGETS
		self.hlfm=QHBoxLayout()
		self.hlop=QHBoxLayout()
		self.vlayout=QVBoxLayout()
		self.glayout=QGridLayout(self)
		self.glayout.addLayout(self.vlayout,0,0)
		
		# FILE MANAGERS
		self.lTFM=QLabel(self.tr("File Manager"))
		self.fontTitle=QFont()
		self.fontTitle.setBold(True)
		self.lTFM.setFont(self.fontTitle)
	        self.line = QFrame()
	        self.line.setFrameShape(QFrame.HLine)
	        self.line.setFrameShadow(QFrame.Plain)
		self.vlayout.addWidget(self.lTFM)
		self.vlayout.addWidget(self.line)

		self.lFM=QLabel(QString(self.tr("File Manager")))
		self.hlfm.addWidget(self.lFM)
		self.cbFM=QComboBox(self)
		i=0
		self.fmpath=[]
		confmanager=loadconf("FileManager")
		currentindex=None
		if os.name == "posix":
			for manager in filemanagers:
				for directory in PATH:
					if os.access(os.path.join(directory, manager), os.X_OK):
						self.cbFM.insertItem(i,QString(manager.capitalize()))
						self.fmpath.append(os.path.join(directory, manager))
						if self.fmpath[i] == confmanager:
							currentindex=i
						i+=1
			if currentindex:
				self.cbFM.setCurrentIndex(currentindex)
		
		elif os.name == "nt":
			manager="explorer.exe"
			self.cbFM.insertItem(0,QString(manager.capitalize()))
			self.fmpath=[manager]

		self.hlfm.addWidget(self.cbFM)
		self.lFMPath=QLabel()
		
		try:
			self.lFMPath.setText(QString(self.fmpath[self.cbFM.currentIndex()]))
		except IndexError:
			pass
		
		self.hlfm.addWidget(self.lFMPath)
		self.vlayout.addLayout(self.hlfm)

		self.cbOpen=QCheckBox(QString(self.tr("Open download directory when all operations finish.")))
		self.vlayout.addWidget(self.cbOpen)
		confopen=loadconf("Open")
		if confopen == "True":
			self.cbOpen.setChecked(True)
		else:
			self.cbOpen.setChecked(False)

		# LANGUAGES
		self.lTLang=QLabel(self.tr("Language"))
		self.lTLang.setFont(self.fontTitle)
	        self.line2 = QFrame()
	        self.line2.setFrameShape(QFrame.HLine)
	        self.line2.setFrameShadow(QFrame.Plain)
		self.vlayout.addWidget(self.lTLang)
		self.vlayout.addWidget(self.line2)
		self.hlang=QHBoxLayout()
		self.vlayout.addLayout(self.hlang)
		self.lLang=QLabel(self.tr("Select language"))
		self.hlang.addWidget(self.lLang)
		self.cbLang=QComboBox()
		lang=loadconf("Language")
		if lang == None or lang == "Default":
			currentlang=0
		listLang=[self.tr("Default")]
		qmfiles=os.listdir(os.path.join(workdir, "locale"))
		qmfiles.sort()
		i=1
		for qmfile in qmfiles:
			if qmfile.endswith(".qm"):
				if lang == qmfile[:-3]:
					currentlang=i
				listLang.append(qmfile[:-3])
				i+=1
		self.cbLang.addItems(listLang)
		self.cbLang.setCurrentIndex(currentlang)
		self.hlang.addWidget(self.cbLang)

		self.lRestart=QLabel(self.tr("(requires restart)"))
		self.hlang.addWidget(self.lRestart)

		self.pbOk=QPushButton(QString(self.tr("OK")))
		self.hlop.addWidget(self.pbOk)
		self.pbCancel=QPushButton(QString(self.tr("Cancel")))
		self.hlop.addWidget(self.pbCancel)
		self.vlayout.addLayout(self.hlop)


		self.connect(self.cbFM, SIGNAL("currentIndexChanged(int)"), self.changepath)
		self.connect(self.pbCancel, SIGNAL("clicked()"), self.close)
		self.connect(self.pbOk, SIGNAL("clicked()"), self.dlgok)

	# SHOW PATH OF MANAGER WHEN SELECTED
	def changepath(self, fmindex):
		self.lFMPath.setText(QString(self.fmpath[fmindex]))

	# WHEN PUSH OK ON DIALOG
	def dlgok(self):
		filemanager=self.fmpath[self.cbFM.currentIndex()]
		openonend=str(self.cbOpen.isChecked())
		saveconf("FileManager", filemanager)
		saveconf("Open", openonend)
		if self.cbLang.currentIndex() == 0:
			saveconf("Language", "Default")
		else:
			saveconf("Language", self.cbLang.currentText())
		self.close()

# LOGIN DLG
class Login(QDialog):
	def __init__(self, parent=None):
		QDialog.__init__(self, parent)

		self.setWindowTitle(QString(self.tr("Login")))

		# WIDGETS
		hluser=QHBoxLayout()
		vlabel=QVBoxLayout()
		vledit=QVBoxLayout()
		hlremember=QHBoxLayout()
		hlbuttons=QHBoxLayout()
		vlogin=QVBoxLayout()
		glogin=QGridLayout(self)
		
		glogin.addLayout(vlogin,0,0)
		lUser=QLabel(QString(self.tr("User")))
		lPass=QLabel(QString(self.tr("Password")))
		vlabel.addWidget(lUser)
		vlabel.addWidget(lPass)
		hluser.addLayout(vlabel)

		self.leUser=QLineEdit()
		self.lePass=QLineEdit()
		self.lePass.setEchoMode(QLineEdit.Password)
		vledit.addWidget(self.leUser)
		vledit.addWidget(self.lePass)
		hluser.addLayout(vledit)

		vlogin.addLayout(hluser)
		
		self.cbGoogle=QCheckBox(QString(self.tr("Login with Google Account")))
		vlogin.addWidget(self.cbGoogle)	

		self.cbRemember=QCheckBox(QString(self.tr("Remember")))
		self.cbRemember.setChecked(True)
		vlogin.addWidget(self.cbRemember)

		pbOk=QPushButton(QString(self.tr("OK")))
		pbCancel=QPushButton(QString(self.tr("Cancel")))
		hlbuttons.addWidget(pbOk)
		hlbuttons.addWidget(pbCancel)
		vlogin.addLayout(hlbuttons)

		# LOAD CONFIGURATION IN ORDER TO PUT IT IN WIDGETS
		User=loadconf("User")
		Password=loadconf("Password")
		Google=loadconf("Google")
		if Google == "True":
			self.cbGoogle.setChecked(True)
		else:
			self.cbGoogle.setChecked(False)
		Remember=loadconf("Remember")
		if Remember == "True":
			self.cbRemember.setChecked(True)
		else:
			self.cbRemember.setChecked(False)

		if User:
			self.leUser.setText(QString(User))
		if Password:
			self.lePass.setText(QString(Password))

		# EMITES WHEN CLOSE DIALOG
		self.connect(pbCancel, SIGNAL("clicked()"), self.close)
		self.connect(pbOk, SIGNAL("clicked()"), self.savelogin)

	# IF OK IN LOGIN DLG, SAVE DATA
	def savelogin(self):
		Google=self.cbGoogle.isChecked()
		Remember=self.cbRemember.isChecked()
		User=self.leUser.text()
		Password=self.lePass.text()
		saveconf("Remember", str(Remember))
		saveconf("User", User)
		saveconf("Password", Password)
		saveconf("Google", str(Google))
		self.close()


# ABOUT DLG
class About(QDialog):
	def __init__(self, parent=None):
		QDialog.__init__(self, parent)
		
		self.setWindowTitle(QString(self.tr("About Elltube")))

		vlayout=QVBoxLayout()

		glayout=QGridLayout(self)
		glayout.addLayout(vlayout,0,0)
		
		# TAB WIDGET
		tabwidget=QTabWidget()
		
		# INFORMATION TAB
		tabwinfo=QWidget()
		infoglayout=QGridLayout(tabwinfo)
		infohlayout=QHBoxLayout()
		# ICON
		pInfo=QPixmap(QString(os.path.join(workdir,"img","elltube.png")))
		plInfo=QLabel()
		plInfo.setFixedSize(50,50)
		plInfo.setScaledContents(True)
		plInfo.setPixmap(pInfo)
		infohlayout.addWidget(plInfo)
		# INFO
		AppName="Elltube %s" % version
		AppDesc=self.tr("YouTube Downloader and Converter.")
		AppCopy=u"Copyright \xa9  2008 Sergio Jovani &lt;<a href=mailto://lesergi@gmail.com>lesergi@gmail.com</a>&gt;"
		AppLic=self.tr("This Software is distributed under the GPL License.")
		AppURL=self.tr("Visit")+" <a href=http://elltube.sourceforge.net>http://elltube.sourceforge.net<a/>"
		lInfo=QLabel(QString(
			"<b>"+AppName+"</b><br><br>"+
			AppDesc+"<br>"+
			AppCopy+"<br>"+
			AppLic+"<br><br>"+
			AppURL
			))
		infohlayout.addWidget(lInfo)
		
		infoglayout.addLayout(infohlayout,0,0)
		
		# TRANSLATORS 
		Translators={
			self.tr("Spanish"): "Sergio Jovani <lesergi@gmail.com>",
			self.tr("Catalan"): "Sergio Jovani <lesergi@gmail.com>",
			self.tr("Italian"): "Francesco Mancuso <mcfrank@tiscali.it>",
			self.tr("Polish"): "Daniel Napora <napcok@gmail.com>",
			self.tr("French"): "David Sagnol <david.sagnol@wanadoo.fr>",
			self.tr("German"): "Andreas Niss <andreasniss@web.de>",
			self.tr("Portuguese (Brazil)"): u"Márcio R. P. Moraes <marciopanto@gmail.com>",
			self.tr("Serbian"): u"Mladen Pejaković <pejakm@gmail.com>"
			}
			
		trans=Translators.keys()
		trans.sort()
		


		tabwtrans=QWidget()
		transglayout=QGridLayout(tabwtrans)
		transhlayout=QHBoxLayout()
		
		pTrans=QPixmap(QString(os.path.join(workdir,"img","translators.png")))
		plTrans=QLabel()
		plTrans.setFixedSize(50,50)
		plTrans.setScaledContents(True)
		plTrans.setPixmap(pTrans)
		transhlayout.addWidget(plTrans)
		
		
		TransList=[]
		
		for tran in trans:
			TransList.append(tran)
			
		
		
		LabelList=[]
		
		for tran in TransList:
			LabelList.append("<b>"+unicode(tran)+"</b>"+": "+Translators[tran].replace("<","&lt;").replace(">","&gt;"))
			


		i=0
		
		for tran in LabelList:
			if i == 0:
				Label=tran+"<br>"
			else:
				Label=Label+" "+tran+"<br>"
			i+=1
		
		lTrans=QLabel(QString(Label))
		transhlayout.addWidget(lTrans)
		
		transglayout.addLayout(transhlayout, 0, 0)
		
		# TAB LICENSE
		tabwlicen=QWidget()
		licenglayout=QGridLayout(tabwlicen)
		licenhlayout=QHBoxLayout()
		# ICON
		pLicen=QPixmap(QString(os.path.join(workdir,"img","license.png")))
		plLicen=QLabel()
		plLicen.setFixedSize(50,50)
		plLicen.setScaledContents(True)
		plLicen.setPixmap(pLicen)
		licenhlayout.addWidget(plLicen)
		# INFO
		lLicen=QTextEdit(QString(self.tr("This program is free software; you can redistribute "+
			"it and/or modify it under the terms of the GNU General Public "+
			"License as published by the Free Software Foundation; either "+
			"version 2 of the License, or (at your option) any later version.")))
		lLicen.setReadOnly(True)
		licenhlayout.addWidget(lLicen)
		
		licenglayout.addLayout(licenhlayout,0,0)
		
		tabwidget.insertTab(0, tabwinfo, QString(self.tr("Information")))
		tabwidget.insertTab(1, tabwtrans, QString(self.tr("Translators")))
		tabwidget.insertTab(2, tabwlicen, QString(self.tr("License")))
		
		
		vlayout.addWidget(tabwidget)
		
		pbok=QPushButton(QString(self.tr("OK")))
		
		vlayout.addWidget(pbok)
		
		self.connect(pbok, SIGNAL("clicked()"), self.close)
		self.connect(lInfo, SIGNAL("linkActivated(QString)"), self.openurl)

	def openurl(self, url):
		webbrowser.open(url)


class OpenLog(QDialog):
	def __init__(self, parent=None):
		QDialog.__init__(self, parent)
		
		self.setWindowTitle(QString(self.tr("Converting failed!")))

		glayout=QGridLayout(self)

		logfile=open(os.path.join(user.home, ".elltube-log"),"r")
		logtext=logfile.read()
		logfile.close()
		tbLog=QTextBrowser()
		tbLog.setText(logtext)
		tbLog.setReadOnly(True)	

		pbOK=QPushButton(self.tr("OK"))

		glayout.addWidget(tbLog,0,0)
		glayout.addWidget(pbOK,1,0)

		self.connect(pbOK, SIGNAL("clicked()"), self.close)

app = QApplication(sys.argv)
translator = QTranslator(app)
qttranslator = QTranslator(app)

langconf=loadconf("Language")

if langconf == None or langconf == "Default":
	lang=locale.getdefaultlocale()[0]
else:
	lang=langconf

try:
	translator.load(lang, os.path.join(workdir, "locale"))
	qttranslator.load("qt_"+lang, QLibraryInfo.location(QLibraryInfo.TranslationsPath))
except:
	pass

app.installTranslator(translator)
app.installTranslator(qttranslator)
window = MainWindow()
app.connect(app, SIGNAL('lastWindowClosed()'), window.savequit)
window.show()
sys.exit(app.exec_())
